unit BOM;

interface

uses
  classes
  , contnrs
  ;

type
  TUserAction = class (TObject)
  private
    FName: string;
    FOID: Integer;
  public
    procedure Execute;
    property Name: string read FName write FName;
    property OID: Integer read FOID write FOID;
  end;
  
  TUserType = class (TObject)
  private
    FActions: TList;
    FName: string;
    FOID: Integer;
  public
    constructor Create;
    destructor Destroy; override;
    property Actions: TList read FActions write FActions;
    property Name: string read FName write FName;
    property OID: Integer read FOID write FOID;
  end;
  
  TUser = class (TObject)
  private
    FFullName: string;
    FOID: Integer;
    FPassword: string;
    FUserName: string;
    FUserType: TUserType;
  public
    property FullName: string read FFullName write FFullName;
    property OID: Integer read FOID write FOID;
    property Password: string read FPassword write FPassword;
    property UserName: string read FUserName write FUserName;
    property UserType: TUserType read FUserType write FUserType;
  end;
  
  TUserActionList = class (TObjectList)
  end;
  
  TUserTypeList = class (TObjectList)
  end;
  
  TUserList = class (TObjectList)
  end;
  
// Define a few global variables (not recommended for production systems!)
var
  UserList: TUserList;
  UserTypeList: TUserTypeList;
  UserActionList: TUserActionList;

implementation

uses
  sysutils
  ;

procedure PopulateObjects;
var
  oid, n, m, nActions: integer;
  usrTemp: TUser;
  usrTypeTemp: TUserType;
  usrActionTemp: TUserAction;
begin
  oid := 0;
  // Create 10 user actions
  UserActionList := TUserActionList.Create;
  for n := 0 to 9 do
  begin
    usrActionTemp := TUserAction.Create;
    usrActionTemp.Name := 'UserAction' + IntToStr(n);
    usrActionTemp.OID := oid;
    Inc(oid);
    UserActionList.Add(usrActionTemp);
  end;
  // Create 10 user types
  UserTypeList := TUserTypeList.Create;
  for n := 0 to 9 do
  begin
    usrTypeTemp := TUserType.Create;
    usrTypeTemp.Name := 'UserType' + IntToStr(n);
    usrTypeTemp.OID := oid;
    Inc(oid);
    // Add some actions
    nActions := Random(UserActionList.Count);
    for m := 0 to nActions do
    begin
      usrActionTemp := TUserAction(UserActionList.Items[Random(UserActionList.Count)]);
      while (usrTypeTemp.Actions.IndexOf(usrActionTemp) <> -1) do
      begin
        usrActionTemp := TUserAction(UserActionList.Items[Random(UserActionList.Count)]);
      end;
      usrTypeTemp.Actions.Add(usrActionTemp);
    end;
    UserTypeList.Add(usrTypeTemp);
  end;
  // Create 10 users
  UserList := TUserList.Create;
  for n := 0 to 9 do
  begin
    usrTemp := TUser.Create;
    usrTemp.FullName := 'A. User' + IntToStr(n);
    usrTemp.OID := oid;
    Inc(oid);
    usrTemp.Password := 'password' + IntToStr(n);
    usrTemp.UserName := 'User' + IntToStr(n);
    usrTemp.UserType := TUserType(UserTypeList.Items[Random(UserTypeList.Count)]);
    UserList.Add(usrTemp);
  end;
end;

{
********************************* TUserAction **********************************
}
procedure TUserAction.Execute;
begin
end;

{
********************************** TUserType ***********************************
}
constructor TUserType.Create;
begin
  inherited Create;
  FActions := TList.Create;
end;

destructor TUserType.Destroy;
begin
  FActions.Free;
  inherited Destroy;
end;

initialization
begin
  PopulateObjects;
end;

end.
